"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateCloudFrontDistribution = exports.tagCloudFrontDistribution = exports.setSimpleAuthBehavior = exports.invalidateCloudfrontCacheWithRetry = exports.invalidateCloudfrontCache = exports.identifyingTag = exports.getCacheInvalidations = exports.findDeployedCloudfrontDistribution = exports.createCloudFrontDistribution = void 0;

var _awsHelper = require("../aws-helper");

var _awsServices = require("../aws-services");

var _lambda = require("../lambda");

var _logger = require("../logger");

var _originAccess = require("./origin-access");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const findDeployedCloudfrontDistribution = async domainName => {
  const distributions = await (0, _awsHelper.getAll)(async (nextMarker, page) => {
    _logger.logger.info(`[CloudFront] 🔍 Searching cloudfront distribution (page ${page})...`);

    const {
      DistributionList
    } = await _awsServices.cloudfront.listDistributions({
      Marker: nextMarker
    }).promise();

    if (!DistributionList) {
      return {
        items: [],
        nextMarker: undefined
      };
    }

    return {
      items: DistributionList.Items || [],
      nextMarker: DistributionList.NextMarker
    };
  });
  const distribution = distributions.find(_distribution => Boolean(_distribution.Aliases.Items && _distribution.Aliases.Items.includes(domainName)));

  if (!distribution) {
    _logger.logger.info(`[CloudFront] 😬 No matching distribution`);

    return null;
  }

  const {
    Tags
  } = await _awsServices.cloudfront.listTagsForResource({
    Resource: distribution.ARN
  }).promise();

  if (!Tags || !Tags.Items || !Tags.Items.find(tag => tag.Key === identifyingTag.Key && tag.Value === identifyingTag.Value)) {
    throw new Error(`CloudFront distribution ${distribution.Id} has no tag ${identifyingTag.Key}:${identifyingTag.Value}`);
  }

  _logger.logger.info(`[CloudFront] 👍 Distribution found: ${distribution.Id}`);

  if (["InProgress", "In Progress"].includes(distribution.Status)) {
    _logger.logger.info(`[CloudFront] ⏱ Waiting for distribution to be deployed. This step might takes up to 25 minutes...`);

    await _awsServices.cloudfront.waitFor("distributionDeployed", {
      Id: distribution.Id
    }).promise();
  }

  return distribution;
};

exports.findDeployedCloudfrontDistribution = findDeployedCloudfrontDistribution;

const tagCloudFrontDistribution = async distribution => {
  _logger.logger.info(`[CloudFront] ✏️ Tagging "${distribution.Id}" bucket with "${identifyingTag.Key}:${identifyingTag.Value}"...`);

  await _awsServices.cloudfront.tagResource({
    Resource: distribution.ARN,
    Tags: {
      Items: [identifyingTag]
    }
  }).promise();
};

exports.tagCloudFrontDistribution = tagCloudFrontDistribution;

const createCloudFrontDistribution = async (domainName, sslCertificateARN) => {
  _logger.logger.info(`[CloudFront] ✏️ Creating Cloudfront distribution with origin "${(0, _awsServices.getS3DomainName)(domainName)}"...`);

  const {
    Distribution
  } = await _awsServices.cloudfront.createDistribution({
    DistributionConfig: getBaseDistributionConfig(domainName, sslCertificateARN)
  }).promise();

  if (!Distribution) {
    throw new Error("[CloudFront] Could not create distribution");
  }

  await tagCloudFrontDistribution(Distribution);

  _logger.logger.info(`[CloudFront] ⏱ Waiting for distribution to be available. This step might takes up to 25 minutes...`);

  await _awsServices.cloudfront.waitFor("distributionDeployed", {
    Id: Distribution.Id
  }).promise();
  return Distribution;
};

exports.createCloudFrontDistribution = createCloudFrontDistribution;

const getBaseDistributionConfig = (domainName, sslCertificateARN) => ({
  CallerReference: Date.now().toString(),
  Aliases: {
    Quantity: 1,
    Items: [domainName]
  },
  Origins: {
    Quantity: 1,
    Items: [{
      Id: (0, _awsServices.getOriginId)(domainName),
      DomainName: (0, _awsServices.getS3DomainName)(domainName),
      CustomOriginConfig: {
        HTTPPort: 80,
        HTTPSPort: 443,
        OriginProtocolPolicy: "http-only",
        OriginSslProtocols: {
          Quantity: 1,
          Items: ["TLSv1"]
        },
        OriginReadTimeout: 30,
        OriginKeepaliveTimeout: 5
      },
      CustomHeaders: {
        Quantity: 0,
        Items: []
      },
      OriginPath: ""
    }]
  },
  Enabled: true,
  Comment: "",
  PriceClass: "PriceClass_All",
  Logging: {
    Enabled: false,
    IncludeCookies: false,
    Bucket: "",
    Prefix: ""
  },
  CacheBehaviors: {
    Quantity: 0
  },
  CustomErrorResponses: {
    Quantity: 0
  },
  Restrictions: {
    GeoRestriction: {
      RestrictionType: "none",
      Quantity: 0
    }
  },
  DefaultRootObject: "index.html",
  WebACLId: "",
  HttpVersion: "http2",
  DefaultCacheBehavior: {
    ViewerProtocolPolicy: "redirect-to-https",
    TargetOriginId: (0, _awsServices.getOriginId)(domainName),
    ForwardedValues: {
      QueryString: false,
      Cookies: {
        Forward: "none"
      },
      Headers: {
        Quantity: 0,
        Items: []
      },
      QueryStringCacheKeys: {
        Quantity: 0,
        Items: []
      }
    },
    AllowedMethods: {
      Quantity: 2,
      Items: ["HEAD", "GET"],
      CachedMethods: {
        Quantity: 2,
        Items: ["HEAD", "GET"]
      }
    },
    TrustedSigners: {
      Enabled: false,
      Quantity: 0
    },
    MinTTL: 0,
    DefaultTTL: 86400,
    MaxTTL: 31536000,
    FieldLevelEncryptionId: "",
    LambdaFunctionAssociations: {
      Quantity: 0,
      Items: []
    },
    SmoothStreaming: false,
    Compress: true // this is required to deliver gzip data

  },
  ViewerCertificate: {
    ACMCertificateArn: sslCertificateARN,
    SSLSupportMethod: "sni-only",
    MinimumProtocolVersion: "TLSv1.1_2016",
    CertificateSource: "acm"
  }
});

const invalidateCloudfrontCache = async (distributionId, paths, wait = false) => {
  _logger.logger.info("[CloudFront] ✏️ Creating invalidation...");

  const {
    Invalidation
  } = await _awsServices.cloudfront.createInvalidation({
    DistributionId: distributionId,
    InvalidationBatch: {
      CallerReference: Date.now().toString(),
      Paths: {
        Quantity: paths.split(",").length,
        Items: paths.split(",").map(path => path.trim())
      }
    }
  }).promise();

  if (!Invalidation) {
    return;
  }

  if (wait) {
    _logger.logger.info("[CloudFront] ⏱ Waiting for invalidation to be completed (can take up to 10 minutes)...");

    await _awsServices.cloudfront.waitFor("invalidationCompleted", {
      DistributionId: distributionId,
      Id: Invalidation.Id
    }).promise();
  }
};

exports.invalidateCloudfrontCache = invalidateCloudfrontCache;

const invalidateCloudfrontCacheWithRetry = async (distributionId, paths, wait = false, count = 0) => {
  try {
    return await invalidateCloudfrontCache(distributionId, paths, wait);
  } catch (error) {
    if (count < 4) {
      return await invalidateCloudfrontCacheWithRetry(distributionId, paths, wait, count + 1);
    }

    throw error;
  }
};

exports.invalidateCloudfrontCacheWithRetry = invalidateCloudfrontCacheWithRetry;
const identifyingTag = {
  Key: "managed-by-aws-spa",
  Value: "v1"
};
exports.identifyingTag = identifyingTag;

const setSimpleAuthBehavior = async (distributionId, lambdaFunctionARN) => {
  const {
    DistributionConfig,
    ETag
  } = await _awsServices.cloudfront.getDistributionConfig({
    Id: distributionId
  }).promise();
  const lambdaConfigs = DistributionConfig.DefaultCacheBehavior.LambdaFunctionAssociations.Items;

  if (lambdaFunctionARN === null) {
    _logger.logger.info(`[CloudFront] 📚 No basic auth configured. Checking if there is a basic auth to remove...`);

    const updatedLambdaFunctions = lambdaConfigs.filter(config => !config.LambdaFunctionARN.includes(_lambda.lambdaPrefix));

    if (updatedLambdaFunctions.length !== lambdaConfigs.length) {
      _logger.logger.info(`[CloudFront] 🗑 Removing lambda function association handling basic auth...`);

      await updateLambdaFunctionAssociations(distributionId, DistributionConfig, updatedLambdaFunctions, ETag);

      _logger.logger.info(`[CloudFront] 👍 Lambda function association removed`);
    } else {
      _logger.logger.info(`[CloudFront] 👍 No basic auth setup`);
    }

    return;
  }

  _logger.logger.info(`[CloudFront] 📚 Checking if basic auth is already setup...`);

  console.log(lambdaConfigs, lambdaFunctionARN);

  if (lambdaConfigs.find(config => config.LambdaFunctionARN === lambdaFunctionARN)) {
    _logger.logger.info(`[CloudFront] 👍 Basic auth already setup`);

    return;
  }

  _logger.logger.info(`[CloudFront] ✏️ Adding simple auth behavior (and replacing "viewer-request" lambda if any)...`);

  await updateLambdaFunctionAssociations(distributionId, DistributionConfig, [...lambdaConfigs.filter(config => config.EventType !== "viewer-request"), {
    LambdaFunctionARN: lambdaFunctionARN,
    EventType: "viewer-request",
    IncludeBody: false
  }], ETag);
};

exports.setSimpleAuthBehavior = setSimpleAuthBehavior;

const getCacheInvalidations = (cacheInvalidations, subFolder) => cacheInvalidations.split(",").map(string => string.trim().replace(/^\//, "")).map(string => subFolder ? `/${subFolder}/${string}` : `/${string}`).join(",");

exports.getCacheInvalidations = getCacheInvalidations;

const updateLambdaFunctionAssociations = async (distributionId, DistributionConfig, lambdaConfigs, ETag) => {
  await _awsServices.cloudfront.updateDistribution({
    Id: distributionId,
    IfMatch: ETag,
    DistributionConfig: _objectSpread(_objectSpread({}, DistributionConfig), {}, {
      DefaultCacheBehavior: _objectSpread(_objectSpread({}, DistributionConfig.DefaultCacheBehavior), {}, {
        LambdaFunctionAssociations: {
          Quantity: lambdaConfigs.length,
          Items: lambdaConfigs
        }
      })
    })
  }).promise();
};

const updateCloudFrontDistribution = async (distributionId, domainName, options) => {
  const {
    shouldBlockBucketPublicAccess,
    oac
  } = options;

  try {
    const {
      DistributionConfig,
      ETag
    } = await _awsServices.cloudfront.getDistributionConfig({
      Id: distributionId
    }).promise();

    if ((0, _originAccess.isRightOriginAlreadyAssociated)(shouldBlockBucketPublicAccess, domainName, DistributionConfig)) {
      return;
    }

    _logger.logger.info(`[Cloudfront] ✏️ Update distribution configuration "${distributionId}"...`);

    await _awsServices.cloudfront.updateDistribution({
      Id: distributionId,
      IfMatch: ETag,
      DistributionConfig: getUpdatedDistributionConfig(domainName, oac?.originAccessControl.Id, DistributionConfig, shouldBlockBucketPublicAccess)
    }).promise();
  } catch (error) {
    throw error;
  }
};

exports.updateCloudFrontDistribution = updateCloudFrontDistribution;

const getUpdatedDistributionConfig = (domainName, originAccessControlId, distributionConfig, shouldBlockBucketPublicAccess) => {
  if (shouldBlockBucketPublicAccess && originAccessControlId) {
    return _objectSpread(_objectSpread({}, distributionConfig), {}, {
      Origins: {
        Quantity: 1,
        Items: [{
          Id: (0, _awsServices.getS3DomainNameForBlockedBucket)(domainName),
          DomainName: (0, _awsServices.getS3DomainNameForBlockedBucket)(domainName),
          OriginAccessControlId: originAccessControlId,
          S3OriginConfig: {
            OriginAccessIdentity: "" //If you're using origin access control (OAC) instead of origin access identity, specify an empty OriginAccessIdentity element

          },
          OriginPath: "",
          CustomHeaders: {
            Quantity: 0,
            Items: []
          }
        }]
      },
      DefaultCacheBehavior: _objectSpread(_objectSpread({}, distributionConfig.DefaultCacheBehavior), {}, {
        TargetOriginId: (0, _awsServices.getS3DomainNameForBlockedBucket)(domainName)
      })
    });
  }

  return _objectSpread(_objectSpread({}, distributionConfig), {}, {
    Origins: {
      Quantity: 1,
      Items: [{
        Id: (0, _awsServices.getOriginId)(domainName),
        DomainName: (0, _awsServices.getS3DomainName)(domainName),
        CustomOriginConfig: {
          HTTPPort: 80,
          HTTPSPort: 443,
          OriginProtocolPolicy: "http-only",
          OriginSslProtocols: {
            Quantity: 1,
            Items: ["TLSv1"]
          },
          OriginReadTimeout: 30,
          OriginKeepaliveTimeout: 5
        },
        CustomHeaders: {
          Quantity: 0,
          Items: []
        },
        OriginPath: ""
      }]
    },
    DefaultCacheBehavior: _objectSpread(_objectSpread({}, distributionConfig.DefaultCacheBehavior), {}, {
      TargetOriginId: (0, _awsServices.getOriginId)(domainName)
    })
  });
};