"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.upsertOriginAccessControl = exports.isRightOriginAlreadyAssociated = exports.getOriginAccessControlName = exports.getExistingOAC = exports.createOAC = exports.cleanExistingOriginAccessControl = void 0;

var _awsServices = require("../aws-services");

var _logger = require("../logger");

const getExistingOAC = async originAccessControlName => {
  const {
    OriginAccessControlList
  } = await _awsServices.cloudfront.listOriginAccessControls().promise();
  const existingOACSummary = OriginAccessControlList?.Items?.find(oac => oac.Name === originAccessControlName);

  if (!existingOACSummary) {
    return null;
  }

  const oac = await _awsServices.cloudfront.getOriginAccessControl({
    Id: existingOACSummary.Id
  }).promise();
  return {
    originAccessControl: oac.OriginAccessControl,
    ETag: oac.ETag
  };
};

exports.getExistingOAC = getExistingOAC;

const createOAC = async (originAccessControlName, domainName, distributionId) => {
  try {
    _logger.logger.info(`[Cloudfront] ✏️ Creating an Origin Access Control for "${domainName}"...`);

    const oac = await _awsServices.cloudfront.createOriginAccessControl({
      OriginAccessControlConfig: {
        Name: originAccessControlName,
        OriginAccessControlOriginType: "s3",
        SigningBehavior: "always",
        SigningProtocol: "sigv4",
        Description: `OAC used by ${domainName} associated to distributionId: ${distributionId}`
      }
    }).promise();
    return {
      originAccessControl: oac.OriginAccessControl,
      ETag: oac.ETag
    };
  } catch (error) {
    throw error;
  }
};

exports.createOAC = createOAC;

const getOriginAccessControlName = (domainName, distributionId) => `${domainName}-${distributionId}`;

exports.getOriginAccessControlName = getOriginAccessControlName;

const upsertOriginAccessControl = async (domainName, distributionId) => {
  const originAccessControlName = getOriginAccessControlName(domainName, distributionId);
  const existingOAC = await getExistingOAC(originAccessControlName);

  if (existingOAC !== null) {
    return existingOAC;
  }

  return await createOAC(originAccessControlName, domainName, distributionId);
};

exports.upsertOriginAccessControl = upsertOriginAccessControl;

const cleanExistingOriginAccessControl = async (domainName, distributionId) => {
  const originAccessControlName = getOriginAccessControlName(domainName, distributionId);
  const existingOAC = await getExistingOAC(originAccessControlName);

  if (existingOAC === null) {
    return;
  }

  await _awsServices.cloudfront.deleteOriginAccessControl({
    Id: existingOAC.originAccessControl.Id,
    IfMatch: existingOAC.ETag
  }).promise();
  return;
};

exports.cleanExistingOriginAccessControl = cleanExistingOriginAccessControl;

const isRightOriginAlreadyAssociated = (shouldBlockBucketPublicAccess, domainName, distributionConfig) => {
  if (shouldBlockBucketPublicAccess) {
    const isOACAlreadyAssociated = distributionConfig?.Origins.Items.find(o => o.Id === (0, _awsServices.getS3DomainNameForBlockedBucket)(domainName));
    return isOACAlreadyAssociated;
  } else {
    const isS3WebsiteAlreadyAssociated = distributionConfig?.Origins.Items.find(o => o.Id === (0, _awsServices.getS3DomainName)(domainName));
    return isS3WebsiteAlreadyAssociated;
  }
};

exports.isRightOriginAlreadyAssociated = isRightOriginAlreadyAssociated;